import {$, db, fm} from 'https://relaydb.us-southeast-1.linodeobjects.com/lib/relaydb-client.mjs'
// import {$, db, fm} from './relaydb-client.mjs'

async function init() {  
  $('#user-full-name').innerHTML = JSON.parse(sessionStorage.profile)._full_name
  const [questions, statuses, sources] = await db.request([
    { select: 'question_id, title, _full_name as author, status_name as status, question_categories, created',
      from: 'question natural join _user natural join status',
      orderBy: 'title'
    },
    { select: 'status_id as value, status_name as label', 
     from: 'status'},
    { select: 'source_id, source_name, source_description',
      from: 'source',}
  ])
  fm.prepSelect('select[name="status_id"]', statuses)

  const tblQuestions = new Tabulator("#tbl-questions", {
    data: questions,
    height: '65vh',
    layout: 'fitColumns',
    selectable: 1,
    // pagination:"local",
    // paginationCounter: function(pageSize, currentRow, currentPage, totalRows, totalPages){
    //     return "Showing " + pageSize +  " rows of " + totalRows + " total";
    // },
    paginationSize: true,
    // paginationSize: 10,
    // paginationSizeSelector:[10, 50, 100, 1000, true],
    pagination: true,
    paginationCounter: 'rows',
    paginationButtonCount: 0,
    rowContextMenu: [
      { label: 'Delete Question', 
        action: async (e, row) => {
          // console.log('row', row.getData())
          const confirmDelete = confirm('Are you sure? Press OK to delete the row.')
          if (confirmDelete) {
            const resp = await db.request({
              deleteFrom: 'question', 
              where: 'question_id = $1',
              bind: [row.getData().question_id]
            })
            console.log('resp', resp)
            row.delete()
          }
        }
      },
      { label: 'Copy Question',
        action: () => alert('Not implemented yet.'),
      }
    ], 
    columnDefaults: {
        tooltip: function(e, cell, onRendered) {
          const row = cell.getRow().getData()
          const query = {
            select: 'question_content',
            from: 'question',
            where: 'question_id = $1',
            bind: [row.question_id],
          }
          const el =document.createElement("div");
          el.innerHTML = 'Loading...'
          // const data = await db.request(query)
          // console.log('got data')
          // el.innerHTML = data[0].question_content
          // return data[0].question_content

          onRendered( async function(){
            const data = await db.request(query)
            console.log('got data')
            el.innerHTML = data[0].question_content
            return data[0].question_content
          })          
          return el
        },
    },
    columns:[
      {title: 'Title', field: 'title', headerFilter: 'input'},
      {title: 'Author', field: 'author', headerFilter:"list", headerFilterParams:{valuesLookup:true, clearable:true}},
      {title: 'Status', field: 'status', headerFilter:"list", headerFilterParams:{valuesLookup:true, clearable:true}},
      {title: 'Categories', field: 'question_categories', headerFilter: 'input'},
      {title: 'Created', field: 'created', headerFilter: 'input', formatter:"datetime", formatterParams:{
        inputFormat:"yyyy-MM-dd",
        outputFormat:"MM/dd/yyyy",
        // invalidPlaceholder:"(invalid date)",
        invalidPlaceholder: d => (new Date(d)).toLocaleDateString(),
        timezone:"America/Los_Angeles",
      }}
    ],
  })
  
  $('#add-question').onclick = async () => {
    const defaultStatus = statuses.filter(r => r.label == 'Proposed')[0].value
    const [newQuestion] = await db.request({
      insertInto: 'question',
      columns: '_username, status_id',
      values: '$1, $2',
      bind: [localStorage.username, defaultStatus],
    })
    newQuestion.author = $('#user-full-name').innerHTML
    fm.populate('#fm-question', newQuestion)
    modalQuestion.show()
  }
  
  const modalQuestion = new bootstrap.Modal(document.getElementById('modal-question'))

  tblQuestions.on("rowSelected", async r => {
    const row = r.getData()
    console.log('row selected', row)
    const query = {
      select: `title, 
        question_content,
        followup,
        choices,
        correct_choice,
        hints,
        private_comments,
        question_categories,
        _full_name as author, -- readonly
        status_id,
        created,
        current_date as last_modified,
        question_id`,
      from: 'question natural join _user',
      where: 'question_id = $1',
      bind: [row.question_id],
    }
    await fm.load('#fm-question', query)
    tblQuestions.deselectRow()
    modalQuestion.show()
  })
  
  $('#btn-save-close-question').onclick = async e => {
    e.preventDefault()
    console.log(await fm.save('#fm-question'), 'rows updated.')
    modalQuestion.hide()
  }
  $('#btn-save-question').onclick = async e => {
    e.preventDefault()
    console.log(await fm.save('#fm-question'), 'rows updated.')
  }
  $('#btn-display-in-game').onclick = async e => {
    e.preventDefault()
    alert('Not implemented yet.')
  }

  $('#modal-question').addEventListener('hidden.bs.modal', function (e) {
    if ($('#fm-question').isDirty) {
      const isConfirmed = confirm('You have unsaved changes. Press OK to continue, Cancel to go back and save.')
      if (!isConfirmed) modalQuestion.show()
    }
  })

  sources.sort( (a,b) => a.source_name < b.source_name ? -1 : 1 )
  
  const tblSources = new Tabulator("#tbl-sources", {
    data: sources,
    height: '400px',
    layout: 'fitColumns',
    selectable: 1,
    // paginationSize: 10,
    // paginationSizeSelector:[10, 50, 100, 1000, true],
    // pagination: true,
    // paginationCounter: 'rows',
    columns:[
      {title: 'Name', field: 'source_name', headerFilter: 'input'},
    ],
  })
    
  // const modalSource = new bootstrap.Modal($('#model-sources'))

  tblSources.on("rowSelected", async r => {
    const row = r.getData()
    console.log('row selected', row)
    $('#source-description').innerHTML = row.source_description
  })
    
}

window.onload = () => {
  console.log('Starting...')
  if (!sessionStorage.session) location.assign('./login.html')
  else init()
}
